# Designed December 2009 by C.plassais + Pilou + Jojo2

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is forbidden:

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name         :   ChrisP_RadialMove
# Original Date   :  03 Juil 2009 version 0.0 (thx to TIG, Thomthom)
# Type         :   Plugin
# Description   :   Exploding movements of groups or/and components
# Usage         :   Right Click - Context Menu
# Version 0.3.2
#---------------------------------------------------------------------------------------
# petit plug d'explosion deplacement" de groupes ou de composants

require 'sketchup.rb'
   
module ChrisP_RadialMove

#Text initialization - This is where to translate
def self.init_text
	#French version
	@mnu_title = "Translation radiale"
	@tip_title = "Applique une translation radiale sur un ensemble de groupes ou composants"
	@msg_no_group = "Vous devez selectionner au moins 2 groupes ou composants"
	@msg_invalid = "Valeurs saisies invalides"
	@msg_no_transfo = "Pas de transformation effective - Rien n'a ete cree"
	
	@lab_dupli = "Copier l'original"
	@lab_cline = "Matrialiser par lignes de construction"
	@lab_pivot = "Coordonnees pivot (defaut = centre) %1"
	@lab_coeff = "Translation en %1 (unit or coeff in %)"
	@lab_groupe_compo = "Groupes et/ou Composants"
	@dlg_yes = "Oui"
	@dlg_no = "Non"
	@dlg_groupe = "Groupes"
	@dlg_composant = "Composants"
	@dlg_les_deux = "Les deux"
	@dlg_title = "Parametres"
	
	#Detremine if length numerics should be with dot or comma
	@dot_separator = Sketchup.parse_length("1.5")
	@tr_id = Geom::Transformation.new
end

#Load the script
def self.load_script
	#initlaize Strings
	init_text
	
	#Add the contextial menu only if the current selection contains a Group
	UI.add_context_menu_handler do |context_menu|
		if check_selection
			context_menu.add_separator
			context_menu.add_item(@mnu_title) { self.top_execute}
		end	
	end	
end

#Check if the selection is valid
def self.check_selection(selection=nil)
	ss = (selection) ? selection : Sketchup.active_model.selection
	lst = ss.find_all { |e| e.instance_of?(Sketchup::Group) || e.instance_of?(Sketchup::ComponentInstance) }
	(lst.length >= 2) ? lst : false
end

#Top execution method (can be called directly with a list of entities)
def self.top_execute(selection=nil)
	#initialization
	@model = Sketchup.active_model
	@entities = @model.active_entities
	selection = @model.selection unless selection
	
	#Checking selection and List of entities
	@lst_grouponents =  check_selection selection
	return UI.messagebox(@msg_no_group) unless @lst_grouponents
	
	#List of groups and list of components
	@lst_groups = @lst_grouponents.find_all { |e| e.instance_of?(Sketchup::Group) }
	@lst_components = @lst_grouponents.find_all { |e| e.instance_of?(Sketchup::ComponentInstance) }
	
	#Center of the selection
	bb = Geom::BoundingBox.new
	@lst_grouponents.each { |gc| bb.add gc.bounds }
	@pivot = bb.center
	
	#Calling the dialog box
	return unless call_dialog

	#Radial translation algorithm
	lltransfo = []
	@lst_grouponents.each do |gc|
		center = gc.bounds.center
		vec = @pivot.vector_to(center)
		vecn = vec.normalize
		new_center = center.clone
		for i in 0..2
			new_center[i] += (@new_coeff[i][1]) ? vec[i] * (@new_coeff[i][0] - 1) : vecn[i] * @new_coeff[i][0]
		end	
		tr = Geom::Transformation.translation center.vector_to(new_center) 
		lltransfo.push [tr, gc, center]
	end
	
	#Verifying that there are at least one real transformation
	unless lltransfo.find { |ltr| ltr[0].to_a != @tr_id.to_a }
		return UI.messagebox(@msg_no_transfo)
	end
	
	#Starting operation
	@model.start_operation @mnu_title
	entities = @model.active_entities
	
	#Adding the pivot contsruction point
	entities.add_cpoint @pivot if @cline
	
	#Adpating the list of elements
	if @dupli
		grp = entities.add_group @lst_grouponents
		grp.copy.explode
		@lst_grouponents = grp.explode
	end
	
	#Radial translation algorithm
	lltransfo.each do |ltr|
		tr, gc, center = ltr
		new_center = tr * center
		gc.transform! tr
		if @cline
			cline = entities.add_cline center, new_center
			cline.stipple = '.'
		end	
	end

	#Putiing the geometry created as a group if duplication
	entities.add_group @lst_grouponents if @dupli

	#Committing operation
	@model.commit_operation
end

#Contruct and show the dialog box
def self.call_dialog
	#Labels
	lab_pivot = ['x', 'y', 'z'].collect { |s| @lab_pivot.sub(/\%1/, s) }
	lab_coeff = ['x', 'y', 'z'].collect { |s| @lab_coeff.sub(/\%1/, s) }
	prompts = [@lab_dupli, @lab_cline] + lab_coeff + lab_pivot
	
	#Current values
	coords = [0, 1, 2].collect { |i| @pivot[i].to_s }
	coeffs = [0.cm, 0.cm, 0.cm].collect { |x| x.to_s }
	val_defaut = [@dlg_yes, @dlg_yes] + coeffs + coords
	
	#Lists for the combo box
	dlg_yes_no = [@dlg_yes, @dlg_no].join '|'
	dlg_groupe_ou_composant = [@dlg_groupe, @dlg_composant, @dlg_les_deux].join '|'
	liste_val = [dlg_yes_no, dlg_yes_no, "", "", "", "", "", ""]
	
	#No need to ask when there are either groups or components
	ngroup = @lst_groups.length
	ncompo = @lst_components.length
	if ngroup > 0 && ncompo > 0
		prompts += [@lab_groupe_compo]
		liste_val += [dlg_groupe_ou_composant]
		val_defaut += [@dlg_les_deux]
	end
	
	#Title
	title = @dlg_title 
	title += " - #{ngroup} " + @dlg_groupe if ngroup > 0
	title += " - #{ncompo} " + @dlg_composant if ncompo > 0
	
	#Showing the dialog box
	results = val_defaut
	while true
		results = UI.inputbox prompts, results, liste_val, title
		return false unless results

		#Checking the results
		dupli, cline,  cfx, cfy, cfz, spx, spy, spz = results
		
		lval = [cfx, cfy, cfz, spx, spy, spz].collect { |s| parse_value s.to_s }
		if lval.find { |v| v[0] == nil }
			UI.messagebox @msg_invalid
			next
		end
		
		#Transfering the values
		@new_coeff = lval[0..2]
		@pivot = Geom::Point3d.new *(lval[3..-1].collect { |v| v[0] })
		@dupli = (dupli == @dlg_yes)
		@cline = (cline == @dlg_yes)
		
		#Keeping groups and/or components
		if ngroup > 0 && ncompo > 0
			case results.last
			when @dlg_groupe
				@lst_grouponents = @lst_groups
			when @dlg_composant
				@lst_grouponents = @lst_components
			end	
		end	
		break
	end	
	
	true
end

#Parse a string as a length (or coefficient with %)
#Returns a pair [value, a] where a is true if in %
#This routine also exchange dot and comma as needed by the local setting
def self.parse_value(s)
	if @dot_separator
		s = s.gsub(/\,\d/) { |c| '.' + c[1..1] }
	else	
		s = s.gsub(/\.\d/) { |c| ',' + c[1..1] }
	end
	if s =~ /(.+)\%/i
		val = $1.to_f
		return((val != 0) ? [val * 0.01, 'true'] :[nil, true])
	end	
	[Sketchup.parse_length(s), false]
end

#Section executed once, when ruby file is loaded
if !file_loaded? File.basename(__FILE__) 
	load_script
	file_loaded File.basename(__FILE__)
end

end	#module ChrisP_RadialMove